package Generics;

import drawing_objects.DrawingPlane;
import movement_strategy.IMoveableObject;

import java.awt.*;

public class PlanesGenericCollection<T extends DrawingPlane, U extends IMoveableObject> {
    /// <summary>
    /// Ширина окна прорисовки
    /// </summary>
    private final int pictureWidth;
    /// <summary>
    /// Высота окна прорисовки
    /// </summary>
    private final int _pictureHeight;
    /// <summary>
    /// Размер занимаемого объектом места (ширина)
    /// </summary>
    private final int _placeSizeWidth = 170;
    /// <summary>
    /// Размер занимаемого объектом места (высота)
    /// </summary>
    private final int _placeSizeHeight = 180;
    /// <summary>
    /// Набор объектов
    /// </summary>
    private final SetGeneric<T> collection;
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="picWidth"></param>
    /// <param name="picHeight"></param>
    public PlanesGenericCollection(int picWidth, int picHeight)
    {
        int width = picWidth / _placeSizeWidth;
        int height = picHeight / _placeSizeHeight;
        pictureWidth = picWidth;
        _pictureHeight = picHeight;
        collection = new SetGeneric<T>(width * height);
    }

    public int size(){
        return collection.getCount();
    }

    public boolean Insert(T obj)
    {
        if (obj == null )
            return false;
        return collection.insert(obj);
    }

    public boolean Remove(int pos) {
        return collection.remove(pos);
    }
    /// <summary>
    /// Получение объекта IMoveableObject
    /// </summary>
    /// <param name="pos"></param>
    /// <returns></returns>
    public U GetU(int pos)
    {
        if(collection.Get(pos) == null)
            return null;
        return (U)collection.Get(pos).GetMoveableObject();
    }
    public T Get(int position){
        if(position < 0 || position >= collection.getCount())
            return null;
        return collection.Get(position);
    }
    /// <summary>
    /// Вывод всего набора объектов
    /// </summary>
    /// <returns></returns>
    public void ShowPlanes(Graphics2D gr)
    {
        DrawBackground(gr);
        DrawObjects(gr);
    }
    /// <summary>
    /// Метод отрисовки фона
    /// </summary>
    /// <param name="g"></param>
    private void DrawBackground(Graphics2D g)
    {
        BasicStroke pen = new BasicStroke(3);
        g.setStroke(pen);
        for (int i = 0; i < pictureWidth / _placeSizeWidth; i++)
        {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
            {//линия разметки места
                g.drawLine(i * _placeSizeWidth + 10, j * _placeSizeHeight +5,
                        i * _placeSizeWidth + _placeSizeWidth / 2 + 50, j * _placeSizeHeight +5);
            }
            g.drawLine(i * _placeSizeWidth + 10, 5, i * _placeSizeWidth + 10,
                    _pictureHeight / _placeSizeHeight * _placeSizeHeight +5);
        }
    }
    /// <summary>
    /// Метод прорисовки объектов
    /// </summary>
    /// <param name="g"></param>
    private void DrawObjects(Graphics2D g)
    {
        for (int i = 0; i < collection.getCount(); i++)
        {
            DrawingPlane plane = collection.Get(i);
            if (plane != null)
            {
                int inRow = pictureWidth / _placeSizeWidth;
                plane.setPosition(pictureWidth - _placeSizeWidth - (i % inRow * _placeSizeWidth)-8, i / inRow * _placeSizeHeight + 20);
                plane.drawTransport(g);
            }
        }
    }
}
